package com.skalup;

import com.vaadin.annotations.Theme;
import com.vaadin.server.Page;
import com.vaadin.server.VaadinRequest;
import com.vaadin.spring.annotation.SpringUI;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.LinkedHashMap;
import java.util.Map;

@Theme("valo")
@SpringUI()
public class MyUI extends UI {

    private static final String EMPTY_URI = "/";

    private final Map<String, Component> pages;

    private Panel springViewDisplay;

    @Autowired
    public MyUI(DefaultView defaultView, UserView scopedView) {

        this.pages = new LinkedHashMap<>();

        pages.put(defaultView.VIEW_NAME, defaultView);
        pages.put(scopedView.VIEW_NAME, scopedView);
    }

    @Override
    protected void init(VaadinRequest request) {
        final VerticalLayout root = new VerticalLayout();
        root.setSizeFull();
        setContent(root);

        final CssLayout navigationBar = new CssLayout();
        navigationBar.addStyleName(ValoTheme.LAYOUT_COMPONENT_GROUP);
        root.addComponent(navigationBar);

        navigationBar.addComponent(createNavigationButton("Home", DefaultView.VIEW_NAME));
        navigationBar.addComponent(createNavigationButton("User", UserView.VIEW_NAME));

        springViewDisplay = new Panel();
        root.addComponentsAndExpand(springViewDisplay);

        initCurrentView();
    }

    private Button createNavigationButton(String caption, final String viewName) {
        Button button = new Button(caption);
        button.addStyleName(ValoTheme.BUTTON_SMALL);
        button.addClickListener(event -> showView(viewName));

        return button;
    }

    private void showView(String viewName) {

        if (viewName.isEmpty()) {
            Page.getCurrent().pushState(EMPTY_URI);
        } else {
            Page.getCurrent().pushState(viewName);
        }

        springViewDisplay.setContent(pages.get(viewName));
    }

    private void initCurrentView() {
        String uri = getUri();

        showView(uri);
    }

    private String getUri() {
        String uri = Page.getCurrent().getUriFragment();

        return uri == null ? "" : uri;
    }
}